﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using System.Reflection;

namespace wpf_bind_command_gestures
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        public Window1()
        {
            InitializeComponent();
        }

        // Bind Ctrl+H and Ctrl+B to the Say Hi menu item and button.
        // Note that a menu item displays only the first gesture as a shortcut.
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            GestureInfo[] gesture_list =
            {
                new GestureInfo(Key.H, ModifierKeys.Control, "Ctrl+H"),
                new GestureInfo(Key.B, ModifierKeys.Control, "Ctrl+B"),
            };
            object[] controls = { mnuFileSayHi, btnSayHi, };
            DefineCommand("SayHiCommand", "Display a message box that says Hi.",
                this, gesture_list, SayHiCommand_CanExecute, SayHiCommand_Executed,
                controls);
        }

        private void mnuFileExit_Click(object sender, RoutedEventArgs e)
        {
            Close();
        }

        private void SayHiCommand_CanExecute(object sender, CanExecuteRoutedEventArgs e)
        {
            // Allow execution if IsChecked is neither false nor indeterminate.
            e.CanExecute = !(chkEnabled.IsChecked ?? false);
        }
        private void SayHiCommand_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            MessageBox.Show("Hi");
        }

        #region CommandBindings

        // Define a gesture's keys and gesture string.
        private class GestureInfo
        {
            internal Key Key;
            internal ModifierKeys Modifier;
            internal string Gesture;

            internal GestureInfo(Key key, ModifierKeys modifier, string gesture)
            {
                Key = key;
                Modifier = modifier;
                Gesture = gesture;
            }
        }

        // Define one command's gesture(s).
        private void DefineCommand(string name, string text,
            UIElement owner, GestureInfo[] gestures,
            CanExecuteRoutedEventHandler can_execute,
            ExecutedRoutedEventHandler executed,
            object[] controls)
        {
            // Create the gesture.
            InputGestureCollection gesture_collection = new InputGestureCollection();

            foreach (GestureInfo info in gestures)
            {
                gesture_collection.Add(new KeyGesture(info.Key, info.Modifier, info.Gesture));
            }

            // Create the command.
            RoutedUICommand command = new RoutedUICommand(text, name,
                owner.GetType(), gesture_collection);

            // Bind the command to its event handlers.
            CommandBinding binding = new CommandBinding(command);
            binding.CanExecute += can_execute;
            binding.Executed += executed;
            owner.CommandBindings.Add(binding);

            // Set Command properties for the command control(s).
            foreach (object control in controls)
            {
                PropertyInfo property_info = control.GetType().GetProperty("Command");
                property_info.SetValue(control, command, null);
            }
        }
        #endregion // CommandBindings
    }
}
